searchData={"items":[{"type":"module","title":"Backstage","doc":"","ref":"Backstage.html"},{"type":"function","title":"Backstage.ack/3","doc":"","ref":"Backstage.html#ack/3"},{"type":"function","title":"Backstage.start_link/1","doc":"","ref":"Backstage.html#start_link/1"},{"type":"function","title":"Backstage.transform/2","doc":"","ref":"Backstage.html#transform/2"},{"type":"protocol","title":"Backstage.Aggregate","doc":"An entity used to enforce business rules and ensure consistent state.\n\nWhen implementing an `Aggregate`, consider what rules are important to the business for the given entity. For example, this is where you may enforce email uniqueness or age restrictions for users if it is important to your business.","ref":"Backstage.Aggregate.html"},{"type":"function","title":"Backstage.Aggregate.handle_command/2","doc":"Validates the incoming command meets business requirements, if any.\n\nReturns an event struct or an error tuple.","ref":"Backstage.Aggregate.html#handle_command/2"},{"type":"function","title":"Backstage.Aggregate.handle_event/2","doc":"Modifies `Aggregate` state based on some or all data from the event.\n\nReturns an updated aggregate struct.","ref":"Backstage.Aggregate.html#handle_event/2"},{"type":"type","title":"Backstage.Aggregate.aggregate/0","doc":"","ref":"Backstage.Aggregate.html#t:aggregate/0"},{"type":"type","title":"Backstage.Aggregate.event/0","doc":"","ref":"Backstage.Aggregate.html#t:event/0"},{"type":"type","title":"Backstage.Aggregate.t/0","doc":"All the types that implement this protocol.","ref":"Backstage.Aggregate.html#t:t/0"},{"type":"protocol","title":"Backstage.Command","doc":"A mechanism for ensuring consistent validation and dispatching of command structs.","ref":"Backstage.Command.html"},{"type":"function","title":"Backstage.Command.dispatch/1","doc":"Performs data validation before building a command struct and dispatching to the router.","ref":"Backstage.Command.html#dispatch/1"},{"type":"function","title":"Backstage.Command.factory/1","doc":"Supports testing with example data to support unit, property, and other tests. A factory implementation also provides a layer of documentation for other developers.\n\nThese factories may be implemented alongside the command struct code or in the `:test` environment depending on your team's preferences.","ref":"Backstage.Command.html#factory/1"},{"type":"function","title":"Backstage.Command.validate/1","doc":"Performs data validation before building a command without dispatching to the router.\n\nThis is useful for providing real-time feedback to users on web interfaces, for example. `Ecto.Changeset` is useful for performing these validations.","ref":"Backstage.Command.html#validate/1"},{"type":"function","title":"What to Validate - Backstage.Command.validate/1","doc":"In traditional CRUD applications, an Ecto Changeset sometimes performs database checks for uniqueness and may even include some business requirements. When validating a command, it's best to cast data to expected types and leave business rule checks to the `Backstage.Aggregate`.","ref":"Backstage.Command.html#validate/1-what-to-validate"},{"type":"type","title":"Backstage.Command.t/0","doc":"All the types that implement this protocol.","ref":"Backstage.Command.html#t:t/0"},{"type":"module","title":"Backstage.EventProducer","doc":"","ref":"Backstage.EventProducer.html"},{"type":"function","title":"Backstage.EventProducer.start_link/1","doc":"","ref":"Backstage.EventProducer.html#start_link/1"},{"type":"protocol","title":"Backstage.Interface.Authorization","doc":"A mechanism for controlling access to some or all commands in an application.\n\n# Order of Operations\n\nAuthorization checks should be performed before dispatching commands.\n\n1. An actor submits a form or presses a button via a Web or API interface.\n2. The application confirms the user is allowed to trigger the command.\n3. If allowed, the command is dispatched upon successful validation.\n\n# Authorization Types\n\nThis protocol supports a wide variety of access controls. An application may perform checks based on the actor's role, a relation to another entity affected by the command, permissions assigned to the actor, and any logic that can be written into a function.\n\n| Authorization Type | Supported? |\n| --- | --- |\n| Global Role-Based Access Control | Yes |\n| Organization-Level Role-Based Access Control | Yes |\n| Relationship-Based Access Control | Yes |\n| Attribute-Based Access Control | Yes |","ref":"Backstage.Interface.Authorization.html"},{"type":"function","title":"Backstage.Interface.Authorization.allow?/3","doc":"For a given command, perform any necessary checks to provide a response.","ref":"Backstage.Interface.Authorization.html#allow?/3"},{"type":"type","title":"Backstage.Interface.Authorization.response/0","doc":"","ref":"Backstage.Interface.Authorization.html#t:response/0"},{"type":"type","title":"Backstage.Interface.Authorization.t/0","doc":"All the types that implement this protocol.","ref":"Backstage.Interface.Authorization.html#t:t/0"},{"type":"extras","title":"README","doc":"- [🚧 Backstage 🚧](#-backstage-)\n- [Goals](#goals)\n- [CQRWho?](#cqrwho)\n  - [The Script](#the-script)\n  - [Business Value](#business-value)\n- [Architecture](#architecture)\n- [TODO](#todo)\n- [Installation](#installation)\n\n---\n\n> On Broadway, the backstage is buzzing with activity.\n>\n> The actors are stepping onto the stage.\n>\n> The crew is pulling the curtains, aiming the lights.\n>\n> The band is moving air.\n>\n> The director is observant and diligent.\n>\n> The audience is transfixed.\n>\n> The show must go on, even when mistakes are made.\n\n---\n\n# 🚧 Backstage 🚧\n\nBackstage is a [CQRS & Event Sourcing](#cqrwho) framework optimized for Elixir applications to take advantage of concurrency and parallelism on the BEAM.\n\n> 👨‍🎤 This repository is currently in an **experimentation phase**. It is not intended to be used in any critical production systems.\n>\n> See [hex.pm](https://hex.pm/packages?search=cqrs&sort=recent_downloads) for a list of available CQRS packages.\n\n# Goals\n\n1. Make Event Sourcing more approachable for CRUD developers.\n2. Leverage battle-tested packages from the Elixir ecosystem for maximum performance.\n3. Establish strong patterns for busy development teams.\n\n# CQRWho?\n\n**Command & Query Responsibility Segregation**, or CQRS, is a pattern for separating how data is written to and read from a system. The benefits of this pattern include:\n\n- More control over database performance based on an application's read/write demands\n- Richer data history, useful for auditing, debugging, and AI model training\n- Clearer boundaries for data as it propagates through a system","ref":"readme.html"},{"type":"extras","title":"The Script - README","doc":"For developers familiar with the **Create Read Update Delete** pattern, or CRUD, CQRS may be intimidating at first. We're already seeing new terms and acronyms, which may be unfamiliar. To understand the value of Event Sourcing and CQRS without digging into the specifics, let's think about a script for a Broadway play.\n\nWhether it's authored by a single person or a result of collaboration, a script provides an order of operations for a play. This includes scene-setting and context, lines for characters, music cues, stage direction, and other technical details. This metaphor illustrates how a script provide a repeatable set of written instructions for recreating a performance.\n\nImagine we only had the last page of a script.\n\n- What would we know?\n- What questions could we answer?\n\nWith traditional CRUD database models, we effectively find ourselves with the last page of the script. Our understanding of the system is limited to the data available in its current state. We have limited ability to answer questions which were not anticipated as the data model was designed or as it evolved. While external tools like logging and tracing allow engineers to perform debugging, these tools are not intended nor ideal for business data analysis.","ref":"readme.html#the-script"},{"type":"extras","title":"Business Value - README","doc":"The greatest value delivered by Event Sourcing & CQRS patterns may be its ability to provide answers to unforeseen questions as a business grows and evolves in surprising ways. For engineers focussed on developer operations, these patterns allow more granular control over write and read performance since incoming data (events) is stored separately from data used by application queries (projections).\n\n# Architecture\n\nBackstage uses the [Broadway](https://hexdocs.pm/broadway/introduction.html) package to run highly concurrent and robust data pipelines.\n\n| Component     | Description                                                | Example                 |\n| ------------- | ---------------------------------------------------------- | ----------------------- |\n| Command       | Instructs system to change its state                       | `%AddTicket{}` struct   |\n| Event         | Represents a historical change in the system's state       | `%TicketAdded{}` struct |\n| Router        | Routes commands to aggregates                              | `TicketRouter` module   |\n| Aggregate     | Performs business logic, updates state for a system entity | `Tickets` module        |\n| Event Handler | Executes custom code for matching events                   | `TicketHandler` module  |\n| Projection    | Persists system state to a readable format                 | `tickets` DB table      |\n\n# TODO\n\n- [ ] Investigate Broadway as an Event Broker\n- [ ] Build EventProducer\n- [ ] Investigate protocols for commands, events, and other components\n\n# Installation\n\nTODO","ref":"readme.html#business-value"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]},"content_type":"text/markdown"}